#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# gui.py
# Maximilian Beeskow
# 22.06.2021
# ----------------------
#
# MODULES
from modules.gui_grid import main

# TEST
if __name__ == "__main__":
    main()


#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# chemistry.py
# Maximilian Beeskow
# 22.04.2022
# ----------------------
#
## MODULES
import re
import numpy as np
#
## CLASSES
class PeriodicSystemOfElements:
    #
    def __init__(self):
        pass
    #
    def get_element_names(self):
        pse_names = ["H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne", "Na", "Mg", "Al", "Si", "P", "S", "Cl",
                         "Ar", "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As",
                         "Se", "Br", "Kr", "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In",
                         "Sn", "Sb", "Te", "I", "Xe", "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb",
                         "Dy", "Ho", "Er", "Tm", "Yb", "Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl",
                         "Pb", "Bi", "Po", "At", "Rn", "Fr", "Ra", "Ac", "Th", "Pa", "U"]
        #
        return  pse_names
    #
    def get_isotope_masses(self):
        pse_masses = [[1.007825, 2.014102, 3.016049]
        ]
        #
        return pse_masses
    #
    def get_isotopic_mcr(self, isotopes):
        pse_names = self.get_element_names()
        mcr_values = np.zeros(len(isotopes))
        for i in range(len(isotopes)):
            key = re.search("(\D+)(\d+)", isotopes)
            index_isotope = pse_names.index(key.group(1))

class PeriodicSystem:
    """ Class that generates the chemical and physical data of the elements"""
    #
    def __init__(self, name=None, atomicnumber=None):
        """
        :param name: name of the chemical element
        :param atomicnumber: atomic number of the chemical element
        """
        self.name = name
        self.atomicnumber = atomicnumber
    #
    def get_data(self):
        data = []
        # [symbol, atomic number, atomic mass, density, bulk modulus, shear modulus, young's modulus, vP,
        # vS, resistivity]
        if self.name in ["H", "Hydrogen", "hydrogen"] or self.atomicnumber == 1:
            mass_molar = 1.008
            density = 0.0899
            bulk_mod = round(1.4*10**(-4), 5)
            shear_mod = 0.0
            young_mod = (9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = np.sqrt((shear_mod*10**9)/(density))
            resistivity = 0.0
            thermal_cond = 0.1805
            ionization_energy_1st = 13.598
            data = ["H", 1, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["He", "Helium", "helium"] or self.atomicnumber == 2:
            mass_molar = 4.0026
            density = 0.1785
            bulk_mod = round(1.01*10**(-5), 7)
            shear_mod = 0.0
            young_mod = (9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = np.sqrt((shear_mod*10**9)/(density))
            resistivity = 0.0
            thermal_cond = 0.1513
            ionization_energy_1st = 24.587
            ionization_energy_2nd = 54.418
            data = ["He", 2, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Li", "Lithium", "lithium"] or self.atomicnumber == 3:
            mass_molar = 6.938
            density = 530
            bulk_mod = 11
            shear_mod = 4.2
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(9.4*10**(-8), 9)
            thermal_cond = 85
            ionization_energy_1st = 5.392
            ionization_energy_2nd = 75.640
            data = ["Li", 3, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Be", "Beryllium", "beryllium"] or self.atomicnumber == 4:
            mass_molar = 9.0122
            density = 1850
            bulk_mod = 130
            shear_mod = 132
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(4.0*10**(-8), 9)
            thermal_cond = 190
            ionization_energy_1st = 9.323
            ionization_energy_2nd = 18.211
            data = ["Be", 4, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["B", "Boron", "boron"] or self.atomicnumber == 5:
            mass_molar = 10.806
            density = 2460
            bulk_mod = 320
            v_p = 16200
            shear_mod = round((3*(density*v_p**2 - bulk_mod*10**9))/(4)*10**(-9), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 10000
            thermal_cond = 27
            ionization_energy_1st = 8.298
            ionization_energy_2nd = 25.155
            data = ["B", 5, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["C", "Carbon", "carbon"] or self.atomicnumber == 6:
            mass_molar = 12.009
            density = 3510
            bulk_mod = 33
            v_p = 18350
            shear_mod = round((3*(density*v_p**2 - bulk_mod*10**9))/(4)*10**(-9), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 1*10**(-5)
            thermal_cond = 140
            ionization_energy_1st = 11.260
            ionization_energy_2nd = 24.383
            data = ["C", 6, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["N", "Nitrogen", "nitrogen"] or self.atomicnumber == 7:
            mass_molar = 14.006
            density = 1.170
            bulk_mod = 2
            v_p = 333.6
            shear_mod = 0.0
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.02583
            ionization_energy_1st = 14.534
            ionization_energy_2nd = 29.601
            data = ["N", 7, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["O", "Oxygen", "oxygen"] or self.atomicnumber == 8:
            mass_molar = 15.999
            density = 1.33
            v_p = 317.5
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            shear_mod = 0.0
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.02658
            ionization_energy_1st = 13.618
            ionization_energy_2nd = 35.121
            data = ["O", 8, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["F", "Flourine", "flourine"] or self.atomicnumber == 9:
            mass_molar = 18.998
            density = 1650
            bulk_mod = 5
            shear_mod = 2
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.0277
            ionization_energy_1st = 17.423
            ionization_energy_2nd = 34.971
            data = ["F", 9, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ne", "Neon", "neon"] or self.atomicnumber == 10:
            mass_molar = 20.180
            density = 0.84
            v_p = 936
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            shear_mod = 0.0
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.0491
            ionization_energy_1st = 21.565
            ionization_energy_2nd = 40.963
            data = ["Ne", 10, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Na", "Sodium", "sodium"] or self.atomicnumber == 11:
            mass_molar = 22.990
            density = 970
            bulk_mod = 6.3
            shear_mod = 3.3
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(4.9*10**(-8), 9)
            thermal_cond = 140
            ionization_energy_1st = 5.139
            ionization_energy_2nd = 47.287
            data = ["Na", 11, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Mg", "Magnesium", "magnesium"] or self.atomicnumber == 12:
            mass_molar = 24.304
            density = 1740
            bulk_mod = 45
            shear_mod = 17
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(4.4*10**(-8), 9)
            thermal_cond = 160
            ionization_energy_1st = 7.646
            ionization_energy_2nd = 15.035
            data = ["Mg", 12, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Al", "Aluminium", "aluminium"] or self.atomicnumber == 13:
            mass_molar = 26.982
            density = 2700
            bulk_mod = 76
            shear_mod = 26
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(2.7*10**(-8), 9)
            thermal_cond = 235
            ionization_energy_1st = 5.986
            ionization_energy_2nd = 18.829
            data = ["Al", 13, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Si", "Silicon", "silicon"] or self.atomicnumber == 14:
            mass_molar = 28.084
            density = 2330
            bulk_mod = 100
            young_mod = 47
            shear_mod = round((3*bulk_mod*young_mod)/(9*bulk_mod - young_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.001
            thermal_cond = 150
            ionization_energy_1st = 8.152
            ionization_energy_2nd = 16.346
            data = ["Si", 14, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["P", "Phosphorus", "phosphorus"] or self.atomicnumber == 15:
            mass_molar = 30.974
            density = 1940
            bulk_mod = 36.09
            shear_mod = 19.86
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10*10**(-8), 9)
            thermal_cond = 0.236
            ionization_energy_1st = 10.487
            ionization_energy_2nd = 19.770
            data = ["P", 15, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["S", "Sulfur", "sulfur"] or self.atomicnumber == 16:
            mass_molar = 32.059
            density = 2060
            bulk_mod = 6
            shear_mod = 4
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 1*10**(15)
            thermal_cond = 0.205
            ionization_energy_1st = 10.360
            ionization_energy_2nd = 23.338
            data = ["S", 16, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Cl", "Chlorine", "chlorine"] or self.atomicnumber == 17:
            mass_molar = 35.446
            density = 2.95
            v_p = 206
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            shear_mod = 0.0
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 100
            thermal_cond = 0.0089
            ionization_energy_1st = 12.968
            ionization_energy_2nd = 23.814
            data = ["Cl", 17, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ar", "Argon", "argon"] or self.atomicnumber == 18:
            mass_molar = 39.948
            density = 1.66
            v_p = 319
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            shear_mod = 0.0
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.01772
            ionization_energy_1st = 15.760
            ionization_energy_2nd = 27.630
            data = ["Ar", 18, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["K", "Potassium", "potassium"] or self.atomicnumber == 19:
            mass_molar = 39.098
            density = 860
            bulk_mod = 3.1
            shear_mod = 1.3
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(7.5*10**(-8), 9)
            thermal_cond = 100
            ionization_energy_1st = 4.341
            ionization_energy_2nd = 31.630
            data = ["K", 19, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ca", "Calcium", "calcium"] or self.atomicnumber == 20:
            mass_molar = 40.078
            density = 1540
            bulk_mod = 17
            shear_mod = 7.4
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(3.4*10**(-8), 9)
            thermal_cond = 200
            ionization_energy_1st = 6.113
            ionization_energy_2nd = 11.872
            data = ["Ca", 20, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Sc", "Scandium", "scandium"] or self.atomicnumber == 21:
            mass_molar = 44.956
            density = 2990
            bulk_mod = 57
            shear_mod = 29
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(55*10**(-8), 9)
            thermal_cond = 16
            ionization_energy_1st = 6.561
            ionization_energy_2nd = 12.800
            data = ["Sc", 21, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ti", "Titanium", "titanium"] or self.atomicnumber == 22:
            mass_molar = 47.867
            density = 4510
            bulk_mod = 110
            shear_mod = 44
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(40*10**(-8), 9)
            thermal_cond = 21.9
            ionization_energy_1st = 6.828
            ionization_energy_2nd = 13.576
            data = ["Ti", 22, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["V", "Vanadium", "vanadium"] or self.atomicnumber == 23:
            mass_molar = 50.942
            density = 6090
            bulk_mod = 160
            shear_mod = 47
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(20*10**(-8), 9)
            thermal_cond = 30.7
            ionization_energy_1st = 6.746
            ionization_energy_2nd = 14.618
            data = ["V", 23, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Cr", "Chromium", "chromium"] or self.atomicnumber == 24:
            mass_molar = 51.996
            density = 7140
            bulk_mod = 160
            shear_mod = 115
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(12.7*10**(-8), 9)
            thermal_cond = 94
            ionization_energy_1st = 6.767
            ionization_energy_2nd = 16.486
            data = ["Cr", 24, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Mn", "Manganese", "manganese"] or self.atomicnumber == 25:
            mass_molar = 54.938
            density = 7440
            bulk_mod = 160
            young_mod = 198
            shear_mod = round((3*bulk_mod*young_mod)/(9*bulk_mod - young_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(144*10**(-8), 9)
            thermal_cond = 7.8
            ionization_energy_1st = 7.434
            ionization_energy_2nd = 15.640
            data = ["Mn", 25, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Fe", "Iron", "iron"] or self.atomicnumber == 26:
            mass_molar = 55.845
            density = 7870
            bulk_mod = 170
            shear_mod = 82
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10*10**(-8), 9)
            thermal_cond = 80
            ionization_energy_1st = 7.902
            ionization_energy_2nd = 16.188
            data = ["Fe", 26, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Co", "Cobalt", "cobalt"] or self.atomicnumber == 27:
            mass_molar = 58.933
            density = 8890
            bulk_mod = 180
            shear_mod = 75
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(6*10**(-8), 9)
            thermal_cond = 100
            ionization_energy_1st = 7.881
            ionization_energy_2nd = 17.084
            data = ["Co", 27, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ni", "Nickel", "nickel"] or self.atomicnumber == 28:
            mass_molar = 58.639
            density = 8910
            bulk_mod = 180
            shear_mod = 76
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(7.2*10**(-8), 9)
            thermal_cond = 91
            ionization_energy_1st = 7.640
            ionization_energy_2nd = 18.169
            data = ["Ni", 28, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Cu", "Copper", "copper"] or self.atomicnumber == 29:
            mass_molar = 63.546
            density = 8920
            bulk_mod = 140
            shear_mod = 48
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(1.72*10**(-8), 10)
            thermal_cond = 400
            ionization_energy_1st = 7.726
            ionization_energy_2nd = 20.292
            data = ["Cu", 29, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Zn", "Zinc", "zinc"] or self.atomicnumber == 30:
            mass_molar = 65.38
            density = 7140
            bulk_mod = 70
            shear_mod = 43
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(6.0*10**(-8), 10)
            thermal_cond = 116
            ionization_energy_1st = 9.394
            ionization_energy_2nd = 17.964
            data = ["Zn", 30, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ga", "Gallium", "gallium"] or self.atomicnumber == 31:
            mass_molar = 69.723
            density = 5910
            bulk_mod = 50
            shear_mod = 35
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(14*10**(-8), 10)
            thermal_cond = 29
            ionization_energy_1st = 5.999
            ionization_energy_2nd = 20.515
            data = ["Ga", 31, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ge", "Germanium", "germanium"] or self.atomicnumber == 32:
            mass_molar = 72.630
            density = 5320
            bulk_mod = 59.93
            shear_mod = 30.47
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(50000*10**(-8), 10)
            thermal_cond = 60
            ionization_energy_1st = 7.899
            ionization_energy_2nd = 15.935
            data = ["Ge", 32, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["As", "Arsenic", "arsenic"] or self.atomicnumber == 33:
            mass_molar = 74.922
            density = 5720
            bulk_mod = 22
            young_mod = 8
            shear_mod = round((3*bulk_mod*young_mod)/(9*bulk_mod - young_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(33*10**(-8), 10)
            thermal_cond = 50
            ionization_energy_1st = 9.789
            ionization_energy_2nd = 18.589
            data = ["As", 33, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Se", "Selenium", "selenium"] or self.atomicnumber == 34:
            mass_molar = 78.971
            density = 4820
            bulk_mod = 8.3
            shear_mod = 3.7
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10*10**(-8), 10)
            thermal_cond = 0.52
            ionization_energy_1st = 9.752
            ionization_energy_2nd = 21.190
            data = ["Se", 34, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Br", "Bromine", "bromine"] or self.atomicnumber == 35:
            mass_molar = 79.901
            density = 3140
            bulk_mod = 1.9
            shear_mod = 1.425 # estimated
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(1*10**18*10**(-8), 10)
            thermal_cond = 0.12
            ionization_energy_1st = 11.814
            ionization_energy_2nd = 21.591
            data = ["Br", 35, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Kr", "Krypton", "krypton"] or self.atomicnumber == 36:
            mass_molar = 83.798
            density = 3.48
            v_p = 1120
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.00943
            ionization_energy_1st = 14.000
            ionization_energy_2nd = 24.360
            data = ["Kr", 36, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Rb", "Rubidium", "rubidium"] or self.atomicnumber == 37:
            mass_molar = 85.468
            density = 1530
            bulk_mod = 2.5
            young_mod = 2.4
            shear_mod = round((3*bulk_mod*young_mod)/(9*bulk_mod - young_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(13.3*10**(-8), 10)
            thermal_cond = 58
            ionization_energy_1st = 4.177
            ionization_energy_2nd = 27.290
            data = ["Rb", 37, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Sr", "Strontium", "strontium"] or self.atomicnumber == 38:
            mass_molar = 87.62
            density = 2630
            shear_mod = 6.1
            poisson = 0.28
            bulk_mod = round((2*shear_mod*(1+poisson))/(3*(1-2*poisson)), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(13.5*10**(-8), 10)
            thermal_cond = 35
            ionization_energy_1st = 5.695
            ionization_energy_2nd = 11.030
            data = ["Sr", 38, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Y", "Yttrium", "yttrium"] or self.atomicnumber == 39:
            mass_molar = 88.906
            density = 4470
            bulk_mod = 41
            shear_mod = 26
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(60*10**(-8), 9)
            thermal_cond = 17.2
            ionization_energy_1st = 6.217
            ionization_energy_2nd = 12.224
            data = ["Y", 39, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Zr", "Zirconium", "zirconium"] or self.atomicnumber == 40:
            mass_molar = 91.224
            density = 6510
            shear_mod = 33
            young_mod = 68
            bulk_mod = round((young_mod*shear_mod)/(3*(3*shear_mod-young_mod)), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(43.3*10**(-8), 9)
            thermal_cond = 22.7
            ionization_energy_1st = 6.634
            ionization_energy_2nd = 13.130
            data = ["Zr", 40, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Nb", "Niobium", "niobium"] or self.atomicnumber == 41:
            mass_molar = 92.906
            density = 8580
            bulk_mod = 170
            shear_mod = 38
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(15.2*10**(-8), 9)
            thermal_cond = 54
            ionization_energy_1st = 6.759
            ionization_energy_2nd = 14.000
            data = ["Nb", 41, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Mo", "Molybdenum", "molybdenum"] or self.atomicnumber == 42:
            mass_molar = 95.95
            density = 10280
            bulk_mod = 230
            shear_mod = 20
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(5.5*10**(-8), 9)
            thermal_cond = 139
            ionization_energy_1st = 7.092
            ionization_energy_2nd = 16.160
            data = ["Mo", 42, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Tc", "Technetium", "technetium"] or self.atomicnumber == 43:
            mass_molar = 97
            density = 11490
            bulk_mod = 281
            shear_mod = 123
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0000185
            thermal_cond = 50.6
            ionization_energy_1st = 7.280
            ionization_energy_2nd = 15.260
            data = ["Tc", 43, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ru", "Ruthenium", "ruthenium"] or self.atomicnumber == 44:
            mass_molar = 101.07
            density = 12450
            bulk_mod = 220
            shear_mod = 173
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(7.1*10**(-8), 9)
            thermal_cond = 120
            ionization_energy_1st = 7.361
            ionization_energy_2nd = 16.760
            data = ["Ru", 44, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Rh", "Rhodium", "rhodium"] or self.atomicnumber == 45:
            mass_molar = 102.91
            density = 12410
            bulk_mod = 380
            shear_mod = 150
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(4.3*10**(-8), 9)
            thermal_cond = 150
            ionization_energy_1st = 7.459
            ionization_energy_2nd = 18.080
            data = ["Rh", 45, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Pd", "Palladium", "palladium"] or self.atomicnumber == 46:
            mass_molar = 106.42
            density = 12020
            bulk_mod = 180
            shear_mod = 44
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10.8*10**(-8), 9)
            thermal_cond = 72
            ionization_energy_1st = 8.337
            ionization_energy_2nd = 19.430
            data = ["Pd", 46, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ag", "Silver", "silver"] or self.atomicnumber == 47:
            mass_molar = 107.87
            density = 10490
            bulk_mod = 100
            shear_mod = 30
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(1.63*10**(-8), 9)
            thermal_cond = 430
            ionization_energy_1st = 7.576
            ionization_energy_2nd = 21.477
            data = ["Ag", 47, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Cd", "Cadmium", "cadmium"] or self.atomicnumber == 48:
            mass_molar = 112.41
            density = 8640
            bulk_mod = 42
            shear_mod = 19
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(7*10**(-8), 9)
            thermal_cond = 97
            ionization_energy_1st = 8.994
            ionization_energy_2nd = 16.908
            data = ["Cd", 48, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["In", "Indium", "indium"] or self.atomicnumber == 49:
            mass_molar = 114.82
            density = 7310
            shear_mod = 4.4
            poisson = 0.4498
            bulk_mod = round((2*shear_mod*(1+poisson))/(3*(1-2*poisson)), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(8*10**(-8), 9)
            thermal_cond = 82
            ionization_energy_1st = 5.786
            ionization_energy_2nd = 18.870
            data = ["In", 49, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Sn", "Tin", "tin"] or self.atomicnumber == 50:
            mass_molar = 118.71
            density = 7290
            bulk_mod = 58
            shear_mod = 18
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(11.5*10**(-8), 9)
            thermal_cond = 66.6
            ionization_energy_1st = 7.344
            ionization_energy_2nd = 14.632
            data = ["Sn", 50, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Sb", "Antimony", "antimony"] or self.atomicnumber == 51:
            mass_molar = 121.76
            density = 6690
            bulk_mod = 42
            shear_mod = 20
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(40*10**(-8), 9)
            thermal_cond = 24
            ionization_energy_1st = 8.608
            ionization_energy_2nd = 16.630
            data = ["Sb", 51, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Te", "Tellurium", "tellurium"] or self.atomicnumber == 52:
            mass_molar = 127.60
            density = 6250
            bulk_mod = 65
            shear_mod = 16
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10000*10**(-8), 9)
            thermal_cond = 3
            ionization_energy_1st = 9.010
            ionization_energy_2nd = 18.600
            data = ["Te", 52, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["I", "Iodine", "iodine"] or self.atomicnumber == 53:
            mass_molar = 126.90
            density = 4940
            bulk_mod = 7.7
            shear_mod = 6.16 # estimated
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10**15*10**(-8), 9)
            thermal_cond = 0.449
            ionization_energy_1st = 10.451
            ionization_energy_2nd = 19.131
            data = ["I", 53, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Xe", "Xenon", "xenon"] or self.atomicnumber == 54:
            mass_molar = 131.29
            density = 4.49
            v_p = 1090
            shear_mod = 0.0
            bulk_mod = round(v_p**2*density*10**3*10**(-9), 3)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = 0.0
            thermal_cond = 0.00569
            ionization_energy_1st = 12.130
            data = ["Xe", 54, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Cs", "Caesium", "caesium"] or self.atomicnumber == 55:
            mass_molar = 132.91
            density = 1900
            bulk_mod = 1.6
            young_mod = 1.7
            shear_mod = round((3*bulk_mod*young_mod)/(9*bulk_mod - young_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(21*10**(-8), 10)
            thermal_cond = 36
            ionization_energy_1st = 3.894
            ionization_energy_2nd = 23.157
            data = ["Cs", 55, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ba", "Barium", "barium"] or self.atomicnumber == 56:
            mass_molar = 137.33
            density = 3650
            bulk_mod = 9.6
            shear_mod = 4.9
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(34*10**(-8), 9)
            thermal_cond = 18
            ionization_energy_1st = 5.212
            ionization_energy_2nd = 10.004
            data = ["Ba", 56, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["La", "Lanthanum", "lanthanum"] or self.atomicnumber == 57:
            mass_molar = 138.91
            density = 6160
            bulk_mod = 28
            shear_mod = 14
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(61.5*10**(-8), 9)
            thermal_cond = 13
            ionization_energy_1st = 5.577
            ionization_energy_2nd = 11.059
            data = ["La", 57, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ce", "Cerium", "cerium"] or self.atomicnumber == 58:
            mass_molar = 140.12
            density = 6770
            bulk_mod = 22
            shear_mod = 14
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(74*10**(-8), 9)
            thermal_cond = 11
            ionization_energy_1st = 5.539
            ionization_energy_2nd = 10.850
            data = ["Ce", 58, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Pr", "Prasaeodymium", "prasaeodymium"] or self.atomicnumber == 59:
            mass_molar = 140.91
            density = 6480
            bulk_mod = 29
            shear_mod = 15
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(70*10**(-8), 9)
            thermal_cond = 13
            ionization_energy_1st = 5.473
            ionization_energy_2nd = 10.550
            data = ["Pr", 59, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Nd", "Neodymium", "neodymium"] or self.atomicnumber == 60:
            mass_molar = 144.24
            density = 7000
            bulk_mod = 32
            shear_mod = 16
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(64.3*10**(-8), 9)
            thermal_cond = 17
            ionization_energy_1st = 5.525
            ionization_energy_2nd = 10.720
            data = ["Nd", 60, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Pm", "Promethium", "promethium"] or self.atomicnumber == 61:
            mass_molar = 145
            density = 7220
            bulk_mod = 33
            shear_mod = 18
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(75*10**(-8), 9)
            thermal_cond = 15
            ionization_energy_1st = 5.582
            ionization_energy_2nd = 10.900
            data = ["Pm", 61, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Sm", "Samarium", "samarium"] or self.atomicnumber == 62:
            mass_molar = 150.36
            density = 7540
            bulk_mod = 38
            shear_mod = 20
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(94*10**(-8), 9)
            thermal_cond = 13
            ionization_energy_1st = 5.644
            ionization_energy_2nd = 11.070
            data = ["Sm", 62, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Eu", "Europium", "europium"] or self.atomicnumber == 63:
            mass_molar = 151.96
            density = 5250
            bulk_mod = 8.3
            shear_mod = 7.9
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(90*10**(-8), 9)
            thermal_cond = 14
            ionization_energy_1st = 5.670
            ionization_energy_2nd = 11.250
            data = ["Eu", 63, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Gd", "Gadolinium", "gadolinium"] or self.atomicnumber == 64:
            mass_molar = 157.25
            density = 7890
            bulk_mod = 38
            shear_mod = 22
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(131*10**(-8), 9)
            thermal_cond = 11
            ionization_energy_1st = 6.150
            ionization_energy_2nd = 12.090
            data = ["Gd", 64, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Tb", "Terbium", "terbium"] or self.atomicnumber == 65:
            mass_molar = 158.93
            density = 8250
            bulk_mod = 38.7
            shear_mod = 22
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(115*10**(-8), 9)
            thermal_cond = 11
            ionization_energy_1st = 5.864
            ionization_energy_2nd = 11.520
            data = ["Tb", 65, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Dy", "Dysprosium", "dysprosium"] or self.atomicnumber == 66:
            mass_molar = 162.50
            density = 8560
            bulk_mod = 41
            shear_mod = 25
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(92.6*10**(-8), 9)
            thermal_cond = 11
            ionization_energy_1st = 5.939
            ionization_energy_2nd = 11.670
            data = ["Dy", 66, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ho", "Holmium", "holmium"] or self.atomicnumber == 67:
            mass_molar = 164.93
            density = 8780
            bulk_mod = 40
            shear_mod = 26
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(81.4*10**(-8), 9)
            thermal_cond = 16
            ionization_energy_1st = 6.022
            ionization_energy_2nd = 11.800
            data = ["Ho", 67, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Er", "Erbium", "erbium"] or self.atomicnumber == 68:
            mass_molar = 167.26
            density = 9050
            bulk_mod = 44
            shear_mod = 28
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(86*10**(-8), 9)
            thermal_cond = 15
            ionization_energy_1st = 6.108
            ionization_energy_2nd = 11.930
            data = ["Er", 68, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Tm", "Thulium", "thulium"] or self.atomicnumber == 69:
            mass_molar = 168.93
            density = 9320
            bulk_mod = 45
            shear_mod = 31
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(67.6*10**(-8), 9)
            thermal_cond = 17
            ionization_energy_1st = 6.184
            ionization_energy_2nd = 12.050
            data = ["Tm", 69, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Yb", "Ytterbium", "ytterbium"] or self.atomicnumber == 70:
            mass_molar = 173.05
            density = 6970
            bulk_mod = 31
            shear_mod = 9.9
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(25*10**(-8), 9)
            thermal_cond = 34.9
            ionization_energy_1st = 6.254
            ionization_energy_2nd = 12.176
            data = ["Yb", 70, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Lu", "Lutetium", "lutetium"] or self.atomicnumber == 71:
            mass_molar = 174.97
            density = 9840
            bulk_mod = 48
            shear_mod = 27
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(58*10**(-8), 9)
            thermal_cond = 16
            ionization_energy_1st = 5.426
            ionization_energy_2nd = 13.900
            data = ["Lu", 71, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Hf", "Hafnium", "hafnium"] or self.atomicnumber == 72:
            mass_molar = 178.48
            density = 13310
            bulk_mod = 110
            shear_mod = 30
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(34*10**(-8), 9)
            thermal_cond = 23
            ionization_energy_1st = 6.825
            ionization_energy_2nd = 15.000
            data = ["Hf", 72, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ta", "Tantalum", "tantalum"] or self.atomicnumber == 73:
            mass_molar = 180.95
            density = 16680
            bulk_mod = 200
            shear_mod = 69
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(13.5*10**(-8), 9)
            thermal_cond = 57
            ionization_energy_1st = 7.550
            data = ["Ta", 73, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["W", "Tungsten", "tungsten"] or self.atomicnumber == 74:
            mass_molar = 183.84
            density = 19260
            bulk_mod = 310
            shear_mod = 161
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(5.4*10**(-8), 9)
            thermal_cond = 174
            ionization_energy_1st = 7.980
            data = ["W", 74, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Re", "Rhenium", "rhenium"] or self.atomicnumber == 75:
            mass_molar = 186.21
            density = 21030
            bulk_mod = 370
            shear_mod = 178
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(18*10**(-8), 9)
            thermal_cond = 48
            ionization_energy_1st = 7.834
            data = ["Re", 75, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Os", "Osmium", "osmium"] or self.atomicnumber == 76:
            mass_molar = 190.23
            density = 22590
            shear_mod = 222
            poisson = 0.25
            bulk_mod = round((2*shear_mod*(1+poisson))/(3*(1-2*poisson)), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(8.1*10**(-8), 9)
            thermal_cond = 88
            ionization_energy_1st = 8.438
            data = ["Os", 76, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Ir", "Iridium", "iridium"] or self.atomicnumber == 77:
            mass_molar = 192.22
            density = 22560
            bulk_mod = 320
            shear_mod = 210
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(4.7*10**(-8), 9)
            thermal_cond = 150
            ionization_energy_1st = 8.967
            data = ["Ir", 77, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Pt", "Platinum", "platinum"] or self.atomicnumber == 78:
            mass_molar = 195.08
            density = 21450
            bulk_mod = 230
            shear_mod = 61
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(10.6*10**(-8), 9)
            thermal_cond = 72
            ionization_energy_1st = 8.959
            ionization_energy_2nd = 18.563
            data = ["Pt", 78, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Au", "Gold", "gold"] or self.atomicnumber == 79:
            mass_molar = 196.97
            density = 19320
            bulk_mod = 220
            shear_mod = 27
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(2.2*10**(-8), 9)
            thermal_cond = 320
            ionization_energy_1st = 9.226
            ionization_energy_2nd = 20.200
            data = ["Au", 79, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Hg", "Mercury", "mercury"] or self.atomicnumber == 80:
            mass_molar = 200.59
            density = 13550
            bulk_mod = 25
            v_p = 1407
            shear_mod = round(0.75*(density*v_p**2 - bulk_mod*10**9)*10**(-9), 1)
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(96*10**(-8), 9)
            thermal_cond = 8.3
            ionization_energy_1st = 10.438
            ionization_energy_2nd = 18.757
            data = ["Hg", 80, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Tl", "Thallium", "thallium"] or self.atomicnumber == 81:
            mass_molar = 204.38
            density = 11850
            bulk_mod = 43
            shear_mod = 2.8
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(15*10**(-8), 9)
            thermal_cond = 46
            ionization_energy_1st = 6.108
            ionization_energy_2nd = 20.428
            data = ["Tl", 81, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Pb", "Lead", "lead"] or self.atomicnumber == 82:
            mass_molar = 207.2
            density = 11340
            bulk_mod = 46
            shear_mod = 5.6
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(21*10**(-8), 9)
            thermal_cond = 35
            ionization_energy_1st = 7.417
            ionization_energy_2nd = 15.032
            data = ["Pb", 82, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Bi", "Bismuth", "bismuth"] or self.atomicnumber == 83:
            mass_molar = 208.98
            density = 9800
            bulk_mod = 31
            shear_mod = 12
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(130*10**(-8), 9)
            thermal_cond = 8
            ionization_energy_1st = 7.286
            ionization_energy_2nd = 16.703
            data = ["Bi", 83, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Po", "Polonium", "polonium"] or self.atomicnumber == 84:
            mass_molar = 209
            density = 9200
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = round(40*10**(-8), 9)
            thermal_cond = 20
            ionization_energy_1st = 8.414
            data = ["Po", 84, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["At", "Astatine", "astatine"] or self.atomicnumber == 85:
            mass_molar = 210
            density = 6400
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 1.7
            ionization_energy_1st = 9.318
            ionization_energy_2nd = 20.000
            data = ["At", 85, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Rn", "Radon", "radon"] or self.atomicnumber == 86:
            mass_molar = 222
            density = 9.23
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 0.00361
            ionization_energy_1st = 10.749
            data = ["Rn", 86, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Fr", "Francium", "francium"] or self.atomicnumber == 87:
            mass_molar = 223
            density = 348
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 15
            ionization_energy_1st = 4.073
            data = ["Fr", 87, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Ra", "Radium", "radium"] or self.atomicnumber == 88:
            mass_molar = 226
            density = 5500
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = round(100*10**(-8), 9)
            thermal_cond = 19
            ionization_energy_1st = 9.318
            ionization_energy_2nd = 10.147
            data = ["Ra", 88, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Ac", "Actinium", "actinium"] or self.atomicnumber == 89:
            mass_molar = 227
            density = 10070
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 12
            ionization_energy_1st = 5.170
            ionization_energy_2nd = 11.750
            data = ["Ac", 89, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Th", "Thorium", "thorium"] or self.atomicnumber == 90:
            mass_molar = 232.04
            density = 11720
            bulk_mod = 54
            shear_mod = 31
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(15*10**(-8), 9)
            thermal_cond = 54
            ionization_energy_1st = 6.307
            ionization_energy_2nd = 11.900
            data = ["Th", 90, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Pa", "Protactinium", "protactinium"] or self.atomicnumber == 91:
            mass_molar = 231.04
            density = 15370
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = round(18*10**(-8), 9)
            thermal_cond = 47
            ionization_energy_1st = 5.890
            data = ["Pa", 91, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["U", "Uranium", "uranium"] or self.atomicnumber == 92:
            mass_molar = 238.03
            density = 18970
            bulk_mod = 100
            shear_mod = 111
            young_mod = round((9*bulk_mod*shear_mod)/(3*bulk_mod + shear_mod), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(28*10**(-8), 9)
            thermal_cond = 27.6
            ionization_energy_1st = 6.194
            ionization_energy_2nd = 10.600
            data = ["U", 92, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Np", "Neptunium", "neptunium"] or self.atomicnumber == 93:
            mass_molar = 237
            density = 20480
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = round(120*10**(-8), 9)
            thermal_cond = 6
            ionization_energy_1st = 6.266
            data = ["Np", 93, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Pu", "Plutonium", "plutonium"] or self.atomicnumber == 94:
            mass_molar = 244
            density = 19740
            shear_mod = 43
            young_mod = 96
            bulk_mod = round((young_mod*shear_mod)/(3*(3*shear_mod-young_mod)), 1)
            v_p = round(np.sqrt((bulk_mod*10**9 + 4/3*shear_mod*10**9)/(density)), 2)
            v_s = round(np.sqrt((shear_mod*10**9)/(density)), 2)
            resistivity = round(150*10**(-8), 9)
            thermal_cond = 6
            ionization_energy_1st = 6.026
            ionization_energy_2nd = 11.200
            data = ["Pu", 94, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        elif self.name in ["Am", "Americium", "americium"] or self.atomicnumber == 95:
            mass_molar = 243
            density = 13670
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 10
            ionization_energy_1st = 5.974
            data = ["Am", 95, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Cm", "Curium", "curium"] or self.atomicnumber == 96:
            mass_molar = 247
            density = 13510
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 10
            ionization_energy_1st = 5.991
            data = ["Cm", 96, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Bk", "Berkelium", "berkelium"] or self.atomicnumber == 97:
            mass_molar = 247
            density = 13250
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 10
            ionization_energy_1st = 6.198
            data = ["Bk", 97, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st]
        elif self.name in ["Cf", "Californium", "californium"] or self.atomicnumber == 98:
            mass_molar = 251
            density = 15100
            bulk_mod = None
            shear_mod = None
            young_mod = None
            v_p = None
            v_s = None
            resistivity = None
            thermal_cond = 10
            ionization_energy_1st = 6.282
            ionization_energy_2nd = 11.800
            data = ["Cf", 98, mass_molar, density, bulk_mod, shear_mod, young_mod, v_p, v_s, resistivity,
                    thermal_cond, ionization_energy_1st, ionization_energy_2nd]
        #
        return data


#!/usr/bin/env python
# -*-coding: utf-8 -*-

#-----------------------------------------------------------------------------------------------------------------------

# Name:		essential_functions.py
# Author:	Maximilian A. Beeskow
# Date:		23.07.2024

#-----------------------------------------------------------------------------------------------------------------------

## MODULES
# external
import re, os, sys, datetime
import tkinter as tk
import numpy as np
import scipy.stats as stats
import tkinter.filedialog as fd
# internal
try:
    from modules import data
    from modules.spike_elimination import two_sided_test_indices, two_sided_test_outliers
except:
    from pysills.modules import data
    from pysills.modules.spike_elimination import two_sided_test_indices, two_sided_test_outliers

## CLASSES
class EssentialDataProcessing:
    #
    def __init__(self, parent):
        self.parent = parent
    #
    def open_csv(self, var_list, var_listbox):
        if "Default_STD_01.csv" in var_list or "Default_SMPL_01.csv" in var_list:
            var_list.clear()
        filename = fd.askopenfilenames(parent=self.parent, filetypes=(("csv files", "*.csv"), ("all files", "*.*")),
                                       initialdir=os.getcwd())
        for i in filename:
            if i not in var_list:
                var_list.append(i)
                file_parts = i.split("/")
                var_listbox.insert(tk.END, file_parts[-1])
        #
        return filename
#
class Essentials:
    def __init__(self, variable):
        self.variable = variable
    #
    def open_txt_file(self):
        f = open(str(self.variable), "r")
        #
        return f
    #
    def find_nearest_time(self, var_t, times, event):
        try:
            time = var_t.get()
            time = time.replace(',', '.')

            x_nearest_start = min(times, key=lambda x: abs(x-float(time)))
            var_t.set(round(x_nearest_start, 8))
            #self.values_time_helper["start"] = x_nearest_start
        except:
            pass
    #
    def merge_times(self):
        sorted_times = sorted(self.variable, key=lambda x: x[0])
        time_index = 0
        #
        for i in sorted_times:
            if i[0] > sorted_times[time_index][1]:
                time_index += 1
                sorted_times[time_index] = i
            else:
                sorted_times[time_index] = [sorted_times[time_index][0], i[1]]
        #
        merged_times = np.array(sorted_times[:time_index+1])
        #
        return merged_times
    #
    def select_radiobutton(self):
        print("Variable:", self.variable.get())
    #
    def change_default_option(self, var_indiv, n_isotopes, lists=False):
        var_default = self.variable
        for i in range(n_isotopes):
            if lists == False:
                var_indiv[i].set(var_default)
            else:
                var_indiv[i][1].set(var_default)
    #
    def change_option(self, index):
        print(self.variable)
        print("Index:", index)
    #
    def select_time_window(self, op):
        part = self.variable
        if part == "BG":
            if self.var_iw_bg.get() != "No Time Intervals":
                key = re.search(r"(\d+\.\d+)" + " - " + r"(\d+\.\d+)", self.var_iw_bg.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_bg[0] = t_id_start
                self.indices_bg[1] = t_id_end
                self.entr_t_start_bg.delete(0, tk.END)
                self.entr_t_start_bg.insert(0, t_start)
                self.entr_t_end_bg.delete(0, tk.END)
                self.entr_t_end_bg.insert(0, t_end)
                #
        elif part == "MAT":
            if self.var_iw_mat.get() != "No Time Intervals":
                key = re.search(r"(\d+\.\d+)" + " - " + r"(\d+\.\d+)", self.var_iw_mat.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_mat[0] = t_id_start
                self.indices_mat[1] = t_id_end
                self.entr_t_start_mat.delete(0, tk.END)
                self.entr_t_start_mat.insert(0, t_start)
                self.entr_t_end_mat.delete(0, tk.END)
                self.entr_t_end_mat.insert(0, t_end)
        elif part == "SIG":
            if self.var_iw_sig.get() != "No Time Intervals":
                key = re.search(r"(\d+\.\d+)" + " - " + r"(\d+\.\d+)", self.var_iw_sig.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_sig[0] = t_id_start
                self.indices_sig[1] = t_id_end
                self.entr_t_start_sig.delete(0, tk.END)
                self.entr_t_start_sig.insert(0, t_start)
                self.entr_t_end_sig.delete(0, tk.END)
                self.entr_t_end_sig.insert(0, t_end)

    def do_grubbs_test2(self, dataset_complete, alpha=0.05, threshold=1000, n_range=3):
        dataset = self.variable

        outlier_indices_pre = two_sided_test_indices(data=dataset, alpha=alpha)
        outlier_values_pre = two_sided_test_outliers(data=dataset, alpha=alpha)
        outlier_indices = []

        for index, value in enumerate(outlier_values_pre):
            if value > threshold:
                outlier_indices.append(outlier_indices_pre[index])

        outlier_indices.sort()

        if len(outlier_indices) > 0:
            print(outlier_indices)

        data_smoothed = []

        for index, value in enumerate(dataset_complete):
            if value > threshold:
                if index in outlier_indices:
                    average_dataset_new = self.determine_surrounded_values(
                        dataset_complete=dataset_complete, index=index)
                    if value in average_dataset_new["All"]:
                        value_corrected = np.min(average_dataset_new["SP"])
                        data_smoothed.append(value_corrected)
                else:
                    data_smoothed.append(value)
            else:
                data_smoothed.append(value)

        return data_smoothed, outlier_indices

    def do_grubbs_test(self, dataset_complete, alpha=0.05, threshold=1000, n_range=3):
        dataset = self.variable

        outlier_indices_pre = two_sided_test_indices(data=dataset, alpha=alpha)
        outlier_indices = outlier_indices_pre
        outlier_indices.sort()

        data_smoothed = dataset_complete.copy()

        for index_outlier in outlier_indices:
            value_outlier = dataset_complete[index_outlier]
            if value_outlier > threshold:
                average_dataset_new = self.determine_surrounded_values(
                    dataset_complete=dataset_complete, index=index_outlier)
                mean = np.mean(average_dataset_new["All"])
                std = np.std(average_dataset_new["All"], ddof=1)
                val_poi = round(abs(value_outlier - mean)/std, 3)
                val_critical = self.calculate_grubbs_critical_value(alpha=alpha, size=len(average_dataset_new["SP"]))
                if val_poi > val_critical:
                    if value_outlier in average_dataset_new["All"]:
                        value_corrected = np.mean(average_dataset_new["SP"])
                        data_smoothed[index_outlier] = round(value_corrected, 2)

        return data_smoothed, outlier_indices

    def calculate_grubbs_critical_value(self, alpha, size):
        t_dist = stats.t.ppf(1 - alpha/(2*size), size - 2)
        numerator = (size - 1)*np.sqrt(np.square(t_dist))
        denominator = np.sqrt(size)*np.sqrt(size - 2 + np.square(t_dist))
        critical_value = numerator/denominator

        return critical_value

    def calculate_grubbs_value(self, dataset_raw):
        std_dev = np.std(dataset_raw, ddof=1)
        avg_y = np.mean(dataset_raw)
        abs_val_minus_avg = abs(np.array(dataset_raw) - avg_y)
        max_of_deviations = max(abs_val_minus_avg)
        max_ind = np.argmax(abs_val_minus_avg)
        Gcal = max_of_deviations/std_dev

        return Gcal, max_ind

    def determine_surrounding_values(self, dataset_complete, index_poi):
        # POI   - Point Of Interest
        # SP    - Surrounding Points
        helper_values = {"POI": 0, "SP": [], "All": []}
        val_poi = dataset_complete[index_poi]
        helper_values["POI"] = val_poi
        n_data = len(dataset_complete)

        if 1 < index_poi and index_poi < n_data - 3:
            val_prev_2 = dataset_complete[index_poi - 2]
            val_prev_1 = dataset_complete[index_poi - 1]
            val_next_1 = dataset_complete[index_poi + 1]
            val_next_2 = dataset_complete[index_poi + 2]
            helper_values["SP"] = [val_prev_2, val_prev_1, val_next_1, val_next_2]
            helper_values["All"] = [val_prev_2, val_prev_1, val_poi, val_next_1, val_next_2]
        elif index_poi == 0:
            val_next_1 = dataset_complete[index_poi + 1]
            val_next_2 = dataset_complete[index_poi + 2]
            helper_values["SP"] = [val_next_1, val_next_2]
            helper_values["All"] = [val_poi, val_next_1, val_next_2]
        elif index_poi == n_data - 1:
            val_prev_2 = dataset_complete[index_poi - 2]
            val_prev_1 = dataset_complete[index_poi - 1]
            helper_values["SP"] = [val_prev_2, val_prev_1]
            helper_values["All"] = [val_prev_2, val_prev_1, val_poi]
        elif index_poi == 1:
            val_prev_1 = dataset_complete[index_poi - 1]
            val_next_1 = dataset_complete[index_poi + 1]
            val_next_2 = dataset_complete[index_poi + 2]
            helper_values["SP"] = [val_prev_1, val_next_1, val_next_2]
            helper_values["All"] = [val_prev_1, val_poi, val_next_1, val_next_2]
        elif index_poi == n_data - 2:
            val_prev_2 = dataset_complete[index_poi - 2]
            val_prev_1 = dataset_complete[index_poi - 1]
            val_next_1 = dataset_complete[index_poi + 1]
            helper_values["SP"] = [val_prev_2, val_prev_1, val_next_1]
            helper_values["All"] = [val_prev_2, val_prev_1, val_poi, val_next_1]

        return helper_values

    def determine_surrounded_values(self, dataset_complete, index, stepsize=4):
        # POI   - Point Of Interest
        # SP    - Surrounding Points
        helper_values = {"POI": 0, "SP": [], "All": []}
        val_poi = dataset_complete[index]
        helper_values["POI"] = val_poi
        helper_values["All"].append(val_poi)
        last_index = len(dataset_complete) - 1

        if index == 0 or index == last_index:
            datapoints = [dataset_complete[1], dataset_complete[2]]
        elif index == last_index:
            datapoints = [dataset_complete[-3], dataset_complete[-2]]
        elif index == 1:
            datapoints = [dataset_complete[0], dataset_complete[2], dataset_complete[3]]
        elif index == last_index - 1:
            datapoints = [dataset_complete[-4], dataset_complete[-3], dataset_complete[-1]]
        else:
            datapoints = [dataset_complete[index - 2], dataset_complete[index - 1], dataset_complete[index + 1],
                          dataset_complete[index + 2]]

        helper_values["SP"].extend(datapoints)
        helper_values["All"].extend(datapoints)

        # for step in range(1, stepsize):
        #     step_before = index - step
        #     step_after = index + step
        #     if step_before >= 0:
        #         helper_values["SP"].append(dataset_complete[step_before])
        #         helper_values["All"].append(dataset_complete[step_before])
        #     if step_after < len(dataset_complete):
        #         helper_values["SP"].append(dataset_complete[step_after])
        #         helper_values["All"].append(dataset_complete[step_after])

        return helper_values

    def find_outlier(self, limit, interval, data_total, isotope, threshold=1000):
        data = self.variable
        n_outl = 0
        indices_outl = []
        data_corrected = []
        helper_interval = np.arange(interval[0], interval[1])

        outlier_indices_pre = two_sided_test_indices(data=data, alpha=limit)
        outlier_values_pre = two_sided_test_outliers(data=data, alpha=limit)

        outlier_indices = []
        for index, value in enumerate(outlier_values_pre):
            if value > threshold:
                outlier_indices.append(outlier_indices_pre[index])
        outlier_indices.sort()
        limit = abs(100 - limit*100)
        helper_outlier = {}

        if len(outlier_indices) > 0:
            for i in outlier_indices:
                if i == 0:
                    data_ref = []
                    data_ref.extend(data[1:3])
                    data_ref = np.min(data_ref)
                elif i == 1:
                    data_ref = []
                    data_ref.extend(data[0:1])
                    data_ref.extend(data[2:4])
                    data_ref = np.min(data_ref)
                elif i >= 2 and i <= len(data)-3:
                    data_ref = []
                    data_ref.extend(data[i-2:i])
                    data_ref.extend(data[i+1:i+3])
                    data_ref = np.min(data_ref)
                elif i == len(data)-2:
                    data_ref = []
                    data_ref.extend(data[i-2:i])
                    data_ref.extend(data[i+1:i+2])
                    data_ref = np.min(data_ref)
                elif i == len(data)-1:
                    data_ref = []
                    data_ref.extend(data[i-2:i])
                    data_ref = np.min(data_ref)

                if isinstance(data, list) or isinstance(data, np.ndarray):
                    data_invest = data[i]
                else:
                    data_invest = data.iloc[i]

                if data_invest >= threshold:
                    if data_invest > data_ref*(1+limit/100) and data_invest > data_ref:
                        n_outl += 1
                        indices_outl.append(helper_interval[i])
                        if i == 0:
                            data_corrected.append(data_ref)
                        elif i == 1:
                            data_corrected.append(data_ref)
                        elif i >= 2 and i <= len(data)-3:
                            data_corrected.append(data_ref)
                        elif i == len(data)-2:
                            data_corrected.append(data_ref)
                        elif i == len(data)-1:
                            data_corrected.append(data_ref)
                        helper_outlier[i] = data_ref
                    elif data_invest < data_ref*(1-limit/100) and data_invest < data_ref:
                        n_outl += 1
                        indices_outl.append(helper_interval[i])
                        if i == 0:
                            data_corrected.append(data_ref)
                        elif i == 1:
                            data_corrected.append(data_ref)
                        elif i >= 2 and i <= len(data)-3:
                            data_corrected.append(data_ref)
                        elif i == len(data)-2:
                            data_corrected.append(data_ref)
                        elif i == len(data)-1:
                            data_corrected.append(data_ref)
                        helper_outlier[i] = data_ref
                    else:
                        data_corrected.append(data_invest)
                else:
                    data_corrected.append(data_invest)

            data_smoothed = np.array(data_total[isotope])

            for index, value_outlier in helper_outlier.items():
                data_smoothed[index] = value_outlier

            data_smoothed.tolist()
        else:
            data_smoothed = np.array(data_total[isotope])
            data_smoothed.tolist()

        return data_smoothed, indices_outl

    def calculate_sensitivity_regression(self, xi_time, var_i, xi_opt):
        xi_time = np.array(xi_time[var_i])
        x = xi_time[:, 0]
        y = xi_time[:, 1]
        A = np.vstack([x, np.ones(len(x))]).T
        m, c = np.linalg.lstsq(A, y, rcond=None)[0] # m*x + c
        xi_opt[var_i] = [m, c]

class EssentialsSRM:
    def __init__(self):
        var_os = sys.platform
        self.var_os = var_os

    def place_srm_values(self, srm_name, srm_dict):
        path = os.getcwd()
        parent = os.path.dirname(path)
        path = os.path.dirname(os.path.realpath(sys.argv[0]))
        path_main = path

        if "/pysills" in path_main:
            path_main = path.replace("/pysills", "")
        elif "/tests" in path_main:
            path_main = path.replace("/tests", "")
        elif "/bin" in path_main:
            path_main = os.path.abspath(__file__)
            path_main = path_main.replace("/modules/essential_functions.py", "")
        elif "Scripts" in path_main:
            path_main = os.path.abspath(__file__)
            path_main = path_main.replace("modules", "")
            path_main = path_main.replace("essential_functions.py", "")
            path_main = path_main[:-1]
        elif "site-packages" in path_main:
            path_main = os.path.abspath(__file__)
            path_main = path_main.replace("modules", "")
            path_main = path_main.replace("essential_functions.py", "")
            path_main = path_main[:-1]

        if "local_packages" in path_main:
            path_main = path_main.replace("local_packages", "local-packages")

        if "site_packages" in path_main:
            path_main = path_main.replace("site_packages", "site-packages")

        list_srm = list(np.array(
            [["NIST 606"], ["NIST 610"], ["NIST 610 (GeoReM)"], ["NIST 610 (Spandler)"], ["NIST 611"],
             ["NIST 611 (GeoReM)"], ["NIST 612"], ["NIST 612 (GeoReM)"], ["NIST 613"], ["NIST 613 (GeoReM)"],
             ["NIST 614"], ["NIST 614 (GeoReM)"], ["NIST 615"], ["NIST 615 (GeoReM)"], ["NIST 616"],
             ["NIST 616 (GeoReM)"], ["NIST 617"], ["NIST 617 (GeoReM)"], ["USGS BCR-2G (GeoReM)"],
             ["USGS GSD-1G (GeoReM)"], ["USGS GSE-1G (GeoReM)"], ["B6"], ["Durango Apatite"], ["Scapolite 17"],
             ["BAM-376"], ["BCR-2G"], ["BL-Q"], ["Br-Glass"], ["GSD-1G (GeoReM)"], ["GSE-1G (GeoReM)"], ["GSE-2G"],
             ["HAL-O"], ["K-Br"], ["MACS-3"], ["Po 724"], ["STDGL-2B2"], ["OU-6 (GeoReM)"]])[:, 0])

        if "pysills" not in path_main:
            path_main = os.path.join(path_main, "pysills")

        if srm_name == "NIST 606":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_606.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 610":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_610.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 610 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_610_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 610 (Spandler)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_610_Spandler.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 611":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_611.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 611 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_611_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 612":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_612.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 612 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_612_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 613":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_613.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 613 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_613_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 614":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_614.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 614 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_614_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 615":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_615.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 615 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_615_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 616":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_616.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 616 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_616_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 617":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_617.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "NIST 617 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "NIST_617_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "USGS BCR-2G (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "USGS_BCR2G_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "USGS GSD-1G (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "USGS_GSD1G_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "USGS GSE-1G (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "USGS_GSE1G_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "B6":
            path_file = os.path.join(path_main, "lib", "srm", "B6.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "Durango Apatite":
            path_file = os.path.join(path_main, "lib", "srm", "Durango_Apatite.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "Scapolite 17":
            path_file = os.path.join(path_main, "lib", "srm", "Scapolite_17.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "BAM-376":
            path_file = os.path.join(path_main, "lib", "srm", "BAM_376.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "BCR-2G":
            path_file = os.path.join(path_main, "lib", "srm", "BCR_2G.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "BL-Q":
            path_file = os.path.join(path_main, "lib", "srm", "BL_Q.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "Br-Glass":
            path_file = os.path.join(path_main, "lib", "srm", "Br_Glass.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "GSD-1G (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "GSD_1G_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "GSE-1G (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "GSE_1G_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "GSE-2G":
            path_file = os.path.join(path_main, "lib", "srm", "GSE_2G.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "HAL-O":
            path_file = os.path.join(path_main, "lib", "srm", "HAL_O.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "K-Br":
            path_file = os.path.join(path_main, "lib", "srm", "K_Br.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "MACS-3":
            path_file = os.path.join(path_main, "lib", "srm", "MACS_3.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "Po 724":
            path_file = os.path.join(path_main, "lib", "srm", "Po_724.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "STDGL-2B2":
            path_file = os.path.join(path_main, "lib", "srm", "STDGL_2B2.csv")
            data_srm = data.general().importSRM(filename=path_file)
        elif srm_name == "OU-6 (GeoReM)":
            path_file = os.path.join(path_main, "lib", "srm", "OU_6_GeoReM.csv")
            data_srm = data.general().importSRM(filename=path_file)
        else:
            srm_name_new = srm_name.replace(" ", "_")
            path_file = os.path.join(path_main, "lib", "srm", srm_name_new)
            data_srm = data.general().importSRM(filename=path_file)

        for item in data_srm:
            srm_dict[srm_name][item[0]] = round(item[1], 4)


#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# plotting.py
# Maximilian Beeskow
# 01.06.2021
# ----------------------
#
# MODULES
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg, NavigationToolbar2Tk)
from matplotlib.figure import Figure
import numpy as np
import tkinter as tk
import matplotlib
import matplotlib.pyplot as plt
matplotlib.use("TkAgg")

# CLASSES
class Plotting:
    def __init__(self, root, data):
        try:
            self.fig.clf()
            self.ax.cla()
            self.canvas.get_tk_widget().pack_forget()
        except AttributeError:
            pass
        self.color_background = "#FFFFFF"
        self.root = root
        self.data = data
        self.times = self.data. iloc[:, 0]

        self.frame_main = tk.Frame(self.root, bg=self.color_background)
        self.frame_main.place(relwidth=0.8, relheight=0.8, relx=0.01, rely=0.01)
        #
        self.frame_main_check = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_check.place(relwidth=0.10, relheight=0.92, relx=0.90, rely=0.08)
        self.frame_main_plot = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_plot.place(relwidth=0.895, relheight=1.0, relx=0.0, rely=0.0)

        self.names = list(self.data.columns.values)
        self.names.pop(0)
        self._states = {name: tk.BooleanVar(name=name) for name in self.names}
        self._values = {name: self.data[name] for name in self.names}
        self.values = np.array([[self.data[name] for name in self.names]])
        self.y_max = np.amax(self.values)

        btn_select_all = tk.Button(self.frame_main, text="Select all", height=1, width=10, padx=2, pady=2,
                                   fg="#22252D", bg="#F1F1EB",
                                   command=lambda: self.cb_select_all(states=self._states))
        btn_select_all.place(relx=0.9, rely=0.0)
        btn_deselect_all = tk.Button(self.frame_main, text="Deselect all", height=1, width=10, padx=2, pady=2,
                                     fg="#22252D", bg="#F1F1EB",
                                     command=lambda: self.cb_deselect_all(states=self._states))
        btn_deselect_all.place(relx=0.9, rely=0.04)

        self.fig = Figure(figsize=(1,2), facecolor="#FFFFFF")
        self.ax = self.fig.add_subplot()

        for name, state in self._states.items():
            state.set(True)
            cb = tk.Checkbutton(master=self.frame_main_check, variable=state, text=name,
                                height=1, width=15, anchor=tk.W, bg=self.color_background)
            cb.pack()
            state.trace_add("write", self._value_changed)
            if state.get():
                self.ax.plot(self.times, self._values[name], label=name, visible=True)
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_ylim(bottom=100, top=1.5*self.y_max)
        self.ax.set_axisbelow(True)
        self.ax.set_xlabel("x", labelpad=0.5)
        self.ax.set_ylabel("f(x)", labelpad=0.5)
        self.ax.legend(fontsize="x-small", framealpha=1.0, bbox_to_anchor=(0.125, 0.02), loc=3, borderaxespad=0,
                  bbox_transform=plt.gcf().transFigure, ncol=int(len(self._states)/2))
        #
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.frame_main_plot)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        self.toolbar = NavigationToolbar2Tk(self.canvas, self.frame_main_plot)
        self.toolbar.config(background=self.color_background)
        self.toolbar._message_label.config(background=self.color_background)
        #self.toolbar.update()
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

    def _value_changed(self, name, _, op):
        self.fig.clf()
        self.ax.cla()
        self.frame_main_plot = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_plot.place(relwidth=0.875, relheight=1.0, relx=0.0, rely=0.0)
        self.fig.clear()
        self.fig = Figure(figsize=(1,2), facecolor="#FFFFFF")
        self.ax = self.fig.add_subplot()
        try:
            self.canvas.get_tk_widget().pack_forget()
        except AttributeError:
            pass
        if self._states[name].get():
            for name, state in self._states.items():
                if state.get():
                    self.ax.plot(self.times, self._values[name], label=name, visible=True)
        else:
            for name, state in self._states.items():
                if state.get():
                    self.ax.plot(self.times, self._values[name], label=name, visible=True)
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_ylim(bottom=100, top=1.5*self.y_max)
        self.ax.set_axisbelow(True)
        self.ax.set_xlabel("x", labelpad=0.5)
        self.ax.set_ylabel("f(x)", labelpad=0.5)
        self.ax.legend(fontsize="x-small", framealpha=1.0, bbox_to_anchor=(0.125, 0.02), loc=3, borderaxespad=0,
                  bbox_transform=plt.gcf().transFigure, ncol=int(len(self._states)/2))
        #
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.frame_main_plot)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)
        self.toolbar = NavigationToolbar2Tk(self.canvas, self.frame_main_plot)
        self.toolbar.config(background=self.color_background)
        self.toolbar._message_label.config(background=self.color_background)
        #self.toolbar.update()
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

    def cb_select_all(self, states):
        for state in states.values():
            state.set(True)
        state.trace_add("write", self._value_changed)
    #
    def cb_deselect_all(self, states):
        for state in states.values():
            state.set(False)
        state.trace_add("write", self._value_changed)


# -*- coding: utf-8 -*-
# Author: Yiping Liu
# Description:
# Version: 1.0
# Last Modified: May 7, 2023

from sympy import *


def find_pair(x, pairs):
    """
    :param x: a tuple
    :param pairs: a list of tuples
    :return: a dict contains a boolean value and the match (if there is).
    """
    pair_length = len(pairs)
    i = 0
    has = False
    target = ''
    while i < pair_length:
        pair = pairs[i]
        if set(x) == set(pair):
            has = True
            target = pair
        i += 1
    return {
        "has": has,
        "target": target
    }


x = symbols('x')


def newton_raphson(x_n, eq, eq_prime, max_iterators):
    tolerance = 10 ** (-6)
    iterate_number = 0

    while iterate_number < max_iterators:
        print("x_{n}: {x_n}".format(n=iterate_number, x_n=float(x_n)))
        iterate_number += 1
        f = eq.subs({x: x_n}).evalf()
        f_prime = eq_prime.subs({x: x_n}).evalf()
        x_n_plus = x_n - f / f_prime

        if abs((x_n_plus - x_n) / x_n) < tolerance:
            return x_n
        x_n = x_n_plus


def get_charge_number(ion):
    """
    get the charge number of an ion (str)
    :param ion: ion name, a string with "+" or "-" sign followed by number of charge
    :return: charge number
    """
    if "+" in ion:
        lis = ion.split("+")
        if lis[1]:
            result = lis[1]
        else:
            result = 1
    elif "-" in ion:
        lis = ion.split("-")
        if lis[1]:
            lis[1] = '-' + lis[1]
            result = lis[1]
        else:
            result = -1
    else:
        result = 0
    return int(result)


def type_of_species(ion):
    """
    determine the type of a ion (string).
    :param ion:
    :return: 0-neutral, 1-cation, -1-anion
    """
    type_value = 0

    charge = get_charge_number(ion)
    if charge == 0:
        type_value = 0
    if charge > 0:
        type_value = 1
    if charge < 0:
        type_value = -1
    return type_value



#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# complex_signals.py
# Maximilian Beeskow
# 17.08.2021
# ----------------------
#
## MODULES
import tkinter as tk
from modules.gui_elements import SimpleElements as SE
from modules import data
import re
import numpy as np
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

## CLASSES
class ComplexSignals:
    #
    def __init__(self, parent, color_bg, signal, isotopes, times, times_iw_bg, times_iw_sig, files_std, files_smpl,
                 file_actual, test_data):
        self.parent = parent
        self.color_bg = color_bg
        self.color_fg = "black"
        self.signal = signal
        self.isotopes = isotopes
        self.times = times
        self.times_iw_bg = times_iw_bg
        self.times_iw_sig = times_iw_sig
        self.files_std = files_std
        self.files_smpl = files_smpl
        self.file_actual = file_actual
        self.test_data = test_data
        #
        self.n_isotopes = len(self.isotopes[:, 0])
        self.measured_isotopes = self.isotopes[:, 0]
        #
        self.index_start_sig = 0
        self.index_end_sig = len(self.times)-1
        self.indices_sig = [self.index_start_sig, self.index_end_sig]
        self.index_start_bg = 0
        self.index_end_bg = len(self.times)-1
        self.indices_bg = [self.index_start_bg, self.index_end_bg]
        #
    def make_complex_signals_window(self):
        # GEOMETRY
        width = int(4*85 + 15 + 9*90 + 140 + 2*150 + 15)
        height = int(2*20 + (self.n_isotopes+3)*30 + 15)
        self.cs_window = tk.Toplevel(self.parent)
        self.cs_window.geometry(str(width)+"x"+str(height))
        self.cs_window.title("Complex Signals")
        print(self.test_data)
        #
    #
    def make_complex_signals_window_SIG1(self):
        #
        ################################################################################################################
        ## VARIABLES ###################################################################################################
        #
        self.var_sig_mu = []
        self.entr_sig_mu = []
        self.var_sig_std = []
        self.entr_sig_std = []
        self.var_file_std_default = tk.StringVar()
        self.var_file_smpl_default = tk.StringVar()
        self.var_file_std_indiv = []
        self.var_file_smpl_indiv = []
        self.option_list_files_std = []
        self.option_list_files_smpl = []
        self.var_rb = tk.IntVar()
        self.var_cb_std = tk.IntVar()
        self.var_cb_smpl = tk.IntVar()
        #
        ################################################################################################################
        ## GEOMETRY ####################################################################################################
        #
        width = int(7*90 + 2*100 + 2*150 + 15)
        height = int(3*20 + (self.n_isotopes+2)*30 + 15)
        self.cs_window = tk.Toplevel(self.parent)
        self.cs_window.geometry(str(width)+"x"+str(height))
        self.cs_window.title("Complex Signals - Segment 1 (Background)")
        #
        for y in range(int(self.n_isotopes + 5)):
            tk.Grid.rowconfigure(self.cs_window, y, weight=1)
        for x in range(12):
            tk.Grid.columnconfigure(self.cs_window, x, weight=1)
        #
        # Rows
        for i in range(0, 3):
            self.cs_window.grid_rowconfigure(i, minsize=20)
        for i in range(3, int(self.n_isotopes+5)):
            self.cs_window.grid_rowconfigure(i, minsize=30)
        self.cs_window.grid_rowconfigure(int(self.n_isotopes+5), minsize=15)
        # Columns
        for i in range(0, 6):
            self.cs_window.grid_columnconfigure(i, minsize=90)
        for i in range(6, 8):
            self.cs_window.grid_columnconfigure(i, minsize=100)
        for i in range(8, 10):
            self.cs_window.grid_columnconfigure(i, minsize=150)
        self.cs_window.grid_columnconfigure(10, minsize=90)
        self.cs_window.grid_columnconfigure(11, minsize=15)
        #
        ################################################################################################################
        ## LABELS ######################################################################################################
        #
        SE(parent=self.cs_window, row_id=0, column_id=0, n_rows=3, n_columns=4, fg=self.color_fg,
           bg=self.color_bg).create_label(text="General Settings")
        SE(parent=self.cs_window, row_id=3, column_id=0, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Time Settings (BG)")
        SE(parent=self.cs_window, row_id=0, column_id=5, n_rows=3, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Isotopes")
        SE(parent=self.cs_window, row_id=0, column_id=6, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Segment 1")
        SE(parent=self.cs_window, row_id=1, column_id=6, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Background BG")
        SE(parent=self.cs_window, row_id=2, column_id=6, fg=self.color_fg, bg=self.color_bg).create_label(text="\u03BC")
        SE(parent=self.cs_window, row_id=2, column_id=7, fg=self.color_fg, bg=self.color_bg).create_label(text="\u03C3")
        SE(parent=self.cs_window, row_id=0, column_id=8, n_rows=2, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Input File")
        SE(parent=self.cs_window, row_id=2, column_id=8, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Standard STD")
        SE(parent=self.cs_window, row_id=2, column_id=9, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Sample SMPL")
        SE(parent=self.cs_window, row_id=0, column_id=10, n_rows=3, fg=self.color_fg,
           bg=self.color_bg).create_label(text="Display")
        #
        SE(parent=self.cs_window, row_id=3, column_id=5, fg=self.color_fg,
           bg=self.color_bg).create_isotope_column(input_isotopes=self.isotopes)
        #
        ################################################################################################################
        ## Buttons #####################################################################################################
        #
        SE(parent=self.cs_window, row_id=3, column_id=2, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_button(text="Advanced Statistics")
        SE(parent=self.cs_window, row_id=4, column_id=2, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_button(text="Export Results")
        #
        ################################################################################################################
        ## Time Windows ################################################################################################
        #
        self.entr_t_start_bg, self.entr_t_end_bg = SE(parent=self.cs_window, row_id=5, column_id=0, fg=self.color_fg,
           bg=self.color_bg).create_time_window_entries(times=self.times, part="BG")
        self.var_iw_bg = SE(parent=self.cs_window, row_id=4, column_id=0, n_columns=2, fg=self.color_fg,
           bg=self.color_bg).create_option_times(part="BG", times_seg=self.times_iw_bg,
                                                 command=lambda op, part="BG": self.select_time_window(part, op))
        #
        ################################################################################################################
        ## Segment 1 ###################################################################################################
        #
        SE(parent=self.cs_window, row_id=3, column_id=6, fg=self.color_fg,
           bg=self.color_bg).create_signal_entries(var_sig_mu=self.var_sig_mu, entr_sig_mu=self.entr_sig_mu,
                                                   var_sig_std=self.var_sig_std, entr_sig_std=self.entr_sig_std,
                                                   input_isotopes=self.isotopes)
        self.var_file_std_indiv, self.option_list_files_std = SE(parent=self.cs_window, row_id=int(3+self.n_isotopes), column_id=8, fg=self.color_fg,
           bg=self.color_bg).create_option_files(type_name="STD", file_selected=self.file_actual,
                                                 file_type=self.files_std, var_file_default=self.var_file_std_default,
                                                 n_isotopes=self.n_isotopes, command=lambda op, type_name="STD": self.select_option_file_default(type_name, op))
        self.var_file_smpl_indiv, self.option_list_files_smpl = SE(parent=self.cs_window, row_id=int(3+self.n_isotopes), column_id=9, fg=self.color_fg,
           bg=self.color_bg).create_option_files(type_name="SMPL", file_selected=self.file_actual,
                                                 file_type=self.files_smpl, var_file_default=self.var_file_smpl_default,
                                                 n_isotopes=self.n_isotopes, command=lambda op, type_name="SMPL": self.select_option_file_default(type_name, op))
        SE(parent=self.cs_window, row_id=3, column_id=8, fg=self.color_fg,
           bg=self.color_bg).create_option_files_column(var_file_indiv=self.var_file_std_indiv,
                                                        option_list=self.option_list_files_std,
                                                        n_isotopes=self.n_isotopes)
        SE(parent=self.cs_window, row_id=3, column_id=9, fg=self.color_fg,
           bg=self.color_bg).create_option_files_column(var_file_indiv=self.var_file_smpl_indiv,
                                                        option_list=self.option_list_files_smpl,
                                                        n_isotopes=self.n_isotopes)
        SE(parent=self.cs_window, row_id=3, column_id=10, fg=self.color_fg,
           bg=self.color_bg).create_radiobutton_column(var_rb=self.var_rb, isotopes=self.isotopes,
                                                       command=lambda var=self.var_rb: self.select_radiobutton(var))
        SE(parent=self.cs_window, row_id=int(3+self.n_isotopes+1), column_id=8, fg=self.color_fg,
           bg=self.color_bg).create_checkbox(var_cb=self.var_cb_std, text="only STD", type_name="STD",
                                             command=lambda var=self.var_cb_std: self.select_checkbox(var))
        SE(parent=self.cs_window, row_id=int(3+self.n_isotopes+1), column_id=9, fg=self.color_fg,
           bg=self.color_bg).create_checkbox(var_cb=self.var_cb_smpl, text="only SMPL", type_name="SMPL",
                                             command=lambda var=self.var_cb_smpl: self.select_checkbox(var))
        #
        ################################################################################################################
        ## Graphics ####################################################################################################
        #
        #
    #
    def select_radiobutton(self, var):
        print("Variable:", var.get(), "--> isotope", self.isotopes[:, 0][var.get()])
    #
    def select_checkbox(self, var):
        print("Variable:", var.get())
    #
    def select_time_window(self, part, op):
        isotopes = self.isotopes[:, 0]
        if part == "BG":
            if self.var_iw_bg.get() != "No Time Intervals":
                key = re.search("(\d+\.\d+)" + " - " + "(\d+\.\d+)", self.var_iw_bg.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_bg[0] = t_id_start
                self.indices_bg[1] = t_id_end
                self.entr_t_start_bg.delete(0, tk.END)
                self.entr_t_start_bg.insert(0, t_start)
                self.entr_t_end_bg.delete(0, tk.END)
                self.entr_t_end_bg.insert(0, t_end)
                #
            for i in range(self.n_isotopes):
                self.fill_signal_entries(var_i=isotopes[i], var_file_std=self.var_file_std_indiv[i].get(),
                                         var_file_smpl=self.var_file_smpl_indiv[i].get(), part="BG", index=i)
        elif part == "MATR":
            if self.var_iw_matr.get() != "No Time Intervals":
                key = re.search("(\d+\.\d+)" + " - " + "(\d+\.\d+)", self.var_iw_matr.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_matr[0] = t_id_start
                self.indices_matr[1] = t_id_end
                self.entr_t_start_matr.delete(0, tk.END)
                self.entr_t_start_matr.insert(0, t_start)
                self.entr_t_end_matr.delete(0, tk.END)
                self.entr_t_end_matr.insert(0, t_end)
        elif part == "INCL":
            if self.var_iw_incl.get() != "No Time Intervals":
                key = re.search("(\d+\.\d+)" + " - " + "(\d+\.\d+)", self.var_iw_incl.get())
                t_start = float(key.group(1))
                t_end = float(key.group(2))
                #
                t_id_start = self.times[self.times == t_start].index[0]
                t_id_end = self.times[self.times == t_end].index[0]
                #
                self.indices_incl[0] = t_id_start
                self.indices_incl[1] = t_id_end
                self.entr_t_start_incl.delete(0, tk.END)
                self.entr_t_start_incl.insert(0, t_start)
                self.entr_t_end_incl.delete(0, tk.END)
                self.entr_t_end_incl.insert(0, t_end)
    #
    def select_option_file_default(self, type_name, op):
        isotopes = self.isotopes[:, 0]
        if type_name == "STD":
            for i in range(self.n_isotopes):
                self.var_file_std_indiv[i].set(self.var_file_std_default.get())
                self.fill_signal_entries(var_i=isotopes[i], var_file_std=self.var_file_std_indiv[i].get(),
                                         var_file_smpl=self.var_file_smpl_indiv[i].get(), part="BG", index=i)
        elif type_name == "SMPL":
            for i in range(self.n_isotopes):
                self.var_file_smpl_indiv[i].set(self.var_file_smpl_default.get())
                self.fill_signal_entries(var_i=isotopes[i], var_file_std=self.var_file_std_indiv[i].get(),
                                         var_file_smpl=self.var_file_smpl_indiv[i].get(), part="BG", index=i)
    #
    def fill_signal_entries(self, var_i, var_file_std, var_file_smpl, part, index):
        files_helper_std = []
        files_helper_smpl = []
        results_helper = []
        all_data = []
        #
        if var_file_std == "All Standards":
            if var_file_std == "All Standards":
                var_file_std = self.option_list_files_std[1:]
                for item_1 in self.files_std:
                    for item_2 in var_file_std:
                        if item_2 in item_1:
                            files_helper_std.append(item_1)
        else:
            for item in self.files_std:
                if var_file_std in item:
                    var_file_std = item
                    files_helper_std.append(item)
        #
        if var_file_smpl == "All Samples":
            if var_file_smpl == "All Samples":
                var_file_smpl = self.option_list_files_smpl[1:]
                for item_1 in self.files_smpl:
                    for item_2 in var_file_smpl:
                        if item_2 in item_1:
                            files_helper_smpl.append(item_1)
        else:
            for item in self.files_smpl:
                if var_file_smpl in item:
                    var_file_smpl = item
                    files_helper_smpl.append(item)
        #
        if part == "BG":
            if self.var_cb_std.get() == 1 and self.var_cb_smpl.get() == 0:
                for file_std in files_helper_std:
                    dataset_std = data.Data(filename=file_std)
                    df_std = dataset_std.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                    #
                    values_i_std = df_std[var_i][self.indices_bg[0]:self.indices_bg[1]+1]
                    if self.measured_isotopes[self.var_rb.get()] == var_i:
                        all_data.extend(values_i_std.values.tolist())
                        #self.make_histo_plot(input_data=all_data, isotope=var_i)
                    #
                    intensity_i_std = values_i_std.mean()
                    results_helper.append(intensity_i_std)
                    #
                self.entr_sig_mu[index][2].delete(0, tk.END)
                self.entr_sig_mu[index][2].insert(0, round(np.mean(results_helper), 8))
                if len(files_helper_std) == 1:
                    self.entr_sig_std[index][2].delete(0, tk.END)
                    self.entr_sig_std[index][2].insert(0, round(0.0, 8))
                else:
                    self.entr_sig_std[index][2].delete(0, tk.END)
                    self.entr_sig_std[index][2].insert(0, round(np.std(results_helper, ddof=1), 8))
            elif self.var_cb_std.get() == 1 and self.var_cb_smpl.get() == 1:
                for file_std in files_helper_std:
                    for file_smpl in files_helper_smpl:
                        dataset_std = data.Data(filename=file_std)
                        df_std = dataset_std.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                        dataset_smpl = data.Data(filename=file_smpl)
                        df_smpl = dataset_smpl.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                        #
                        values_i_std = df_std[var_i][self.indices_bg[0]:self.indices_bg[1]+1]
                        values_i_smpl = df_smpl[var_i][self.indices_bg[0]:self.indices_bg[1]+1]
                        if self.measured_isotopes[self.var_rb.get()] == var_i:
                            all_data.extend(values_i_std.values.tolist())
                            all_data.extend(values_i_smpl.values.tolist())
                            #self.make_histo_plot(input_data=all_data, isotope=var_i)
                        #
                        intensity_i_std = values_i_std.mean()
                        intensity_i_smpl = values_i_smpl.mean()
                        results_helper.append(intensity_i_std)
                        results_helper.append(intensity_i_smpl)
                        #
                self.entr_sig_mu[index][2].delete(0, tk.END)
                self.entr_sig_mu[index][2].insert(0, round(np.mean(results_helper), 8))
                self.entr_sig_std[index][2].delete(0, tk.END)
                self.entr_sig_std[index][2].insert(0, round(np.std(results_helper, ddof=1), 8))
            elif self.var_cb_std.get() == 0 and self.var_cb_smpl.get() == 1:
                for file_smpl in files_helper_smpl:
                    dataset_smpl = data.Data(filename=file_smpl)
                    df_smpl = dataset_smpl.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
                    #
                    values_i_smpl = df_smpl[var_i][self.indices_bg[0]:self.indices_bg[1]+1]
                    if self.measured_isotopes[self.var_rb.get()] == var_i:
                        all_data.extend(values_i_smpl.values.tolist())
                    #
                    intensity_i_smpl = values_i_smpl.mean()
                    results_helper.append(intensity_i_smpl)
                    #
                self.entr_sig_mu[index][2].delete(0, tk.END)
                self.entr_sig_mu[index][2].insert(0, round(np.mean(results_helper), 8))
                if len(files_helper_smpl) == 1:
                    self.entr_sig_std[index][2].delete(0, tk.END)
                    self.entr_sig_std[index][2].insert(0, round(0.0, 8))
                else:
                    self.entr_sig_std[index][2].delete(0, tk.END)
                    self.entr_sig_std[index][2].insert(0, round(np.std(results_helper, ddof=1), 8))
        #
        if self.measured_isotopes[self.var_rb.get()] == var_i:
            self.make_histo_plot(input_data=all_data, isotope=var_i)
    #
    def make_histo_plot(self, input_data, isotope):
        #
        try:
            self.fig_histo.clf()
            self.ax_histo.cla()
            self.canvas_histo.get_tk_widget().forget_grid()
        except AttributeError:
            pass

        try:
            if self.canvas_histo:
                self.canvas_histo.destroy()
        except AttributeError:
            pass
        #
        self.canvas_histo = None
        self.fig_histo = Figure(facecolor=self.color_bg)
        self.ax_histo = self.fig_histo.add_subplot()
        #
        if self.var_cb_std.get() == 1 and self.var_cb_smpl.get() == 0:
            color = "tomato"
        elif self.var_cb_std.get() == 0 and self.var_cb_smpl.get() == 1:
            color = "forestgreen"
        elif self.var_cb_std.get() == 1 and self.var_cb_smpl.get() == 1:
            color = "slateblue"
        #
        self.ax_histo.axvline(x=np.mean(input_data), color="black", linestyle="dashed")
        self.ax_histo.hist(input_data, bins=15, color=color, edgecolor="black")
        self.ax_histo.grid(True)
        self.ax_histo.set_axisbelow(True)
        self.ax_histo.set_ylabel("Frequency", labelpad=0.5)
        self.ax_histo.set_title("Background intensities of "+str(isotope))
        #
        self.canvas_histo = FigureCanvasTkAgg(self.fig_histo, master=self.cs_window)
        self.canvas_histo.get_tk_widget().grid(row=7, column=0, rowspan=20, columnspan=5, sticky="nesw")
    #